﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;          // added to include serial ports
using System.IO;
using System.Threading;
using DWORD = System.UInt32;

enum cmds
{
    DONE = 0xb0, RESET_FPGA, RELEASE_FPGA, CHECK_EXIST, GET_ID, ERASE_CHIP, ERASE_64K, PROG_PAGE, READ_PAGE, VERIFY_PAGE, POWERUP, POWEROFF
};

namespace iceBliProg
{
    public partial class iceBliProg : Form
    {
        static SerialPort USB_PORT;
        byte[] SerBuf = new byte[300];
        const Int32 PROGSIZE = 1048576;
        byte[] ProgBuf = new byte[PROGSIZE];
        FileStream fs;

        public iceBliProg()
        {
            InitializeComponent();
            USB_PORT = new SerialPort();

            foreach (string s in SerialPort.GetPortNames())        // Get a list of available serial port names.
            {
                comboBox_comport.Items.Add(s);                      // places each "COMx" name into combobox
            }

            FileName.Text = Application.UserAppDataRegistry.GetValue("FileName", "Select File").ToString();
            comboBox_comport.Text = Application.UserAppDataRegistry.GetValue("ComPort", "Select COM Port").ToString(); 
            offsetTxt.Text = Application.UserAppDataRegistry.GetValue("Offset", "0").ToString();
            if (Application.UserAppDataRegistry.GetValue("Verify", "1").ToString() == "1") verify.Checked = true;
            else verify.Checked = false;
            if (Application.UserAppDataRegistry.GetValue("PowerHold", "0").ToString() == "1") powerHold.Checked = true;
            else powerHold.Checked = false;
        }

        private void comboBox_comport_SelectedIndexChanged(object sender, EventArgs e)
        {
            USB_PORT.Close();                            // close any existing handle
            USB_PORT.PortName = comboBox_comport.Text;    // retrieves "COMx" from selection in combo box
            USB_PORT.Parity = 0;
            USB_PORT.BaudRate = 19200;
            USB_PORT.StopBits = StopBits.Two;
            USB_PORT.DataBits = 8;
            USB_PORT.ReadTimeout = 5000;
            USB_PORT.WriteTimeout = 5000;
            USB_PORT.Open();
            Application.UserAppDataRegistry.SetValue("ComPort", comboBox_comport.Text);
        }

        private void transmit(int write_bytes)
        {
            try
            {
                USB_PORT.Write(SerBuf, 0, write_bytes);      // writes specified amount of SerBuf out on COM port
            }
            catch (Exception)
            {
            }
        }


        private void recieve(int read_bytes)
        {
            int x;

            for (x = 0; x < read_bytes; x++)       // this will call the read function for the passed number times, 
            {                                      // this way it ensures each byte has been correctly recieved while
                try                                // still using timeouts
                {
                    USB_PORT.Read(SerBuf, x, 1);     // retrieves 1 byte at a time and places in SerBuf at position x
                }
                catch (Exception)                   // timeout or other error occured, set lost comms indicator
                {
                    SerBuf[0] = 255;
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            OpenFileDialog fdlg = new OpenFileDialog();
            fdlg.Title = "Select  File to Program.";
            fdlg.InitialDirectory = @"c:\";
            fdlg.Filter = "Bin files (*.bin)|*.bin|All files (*.*)|*.*";
            fdlg.FilterIndex = 2;
            fdlg.RestoreDirectory = true;
            if (fdlg.ShowDialog() == DialogResult.OK)
            {
                FileName.Text = fdlg.FileName;
                Application.UserAppDataRegistry.SetValue("FileName", fdlg.FileName);
            }
        }

        private void releaseFPGA(bool hold)                                // run the FPGA
        {
            SerBuf[0] = (byte) cmds.RELEASE_FPGA;
            if (hold == true) SerBuf[1] = 1;                                      // power control, 0 to power off
            else SerBuf[1] = 0;
            transmit(2);
            recieve(1);
        }

        private void powerOff()
        {
            SerBuf[0] = (byte)cmds.POWEROFF; // to power off
            transmit(1);
            recieve(1);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            // read user's offset value
            int offset;
            try
            {
                offset = (int) Convert.ToUInt32(offsetTxt.Text,16);
            }
            catch
            {
                status.Text = "Error - offset format!";
                return;
            }

            // Open the file to program
            try { fs.Close(); }
            catch { }
            try
            {
                fs = File.OpenRead(FileName.Text);
            }
            catch
            {
                status.Text = "Error - Unable to open File!";
                return;
            }
            int len = (int)fs.Length;
            fs.Read(ProgBuf, 0, len);

            // Powerup
            SerBuf[0] = (byte) cmds.POWERUP ;
            SerBuf[1] = 0;
            transmit(2);
            recieve(1);
            Thread.Sleep(500);

            // reset FPGA
            SerBuf[0] = (byte) cmds.RESET_FPGA;
            SerBuf[1] = 0;
            transmit(2);
            Thread.Sleep(50);       // 50mS to ensure last USB packet has gone

            // check if iceBlip Module exists
            SerBuf[0] = (byte) cmds.CHECK_EXIST;
            SerBuf[1] = 0;
            transmit(2);
            recieve(1);
            if(SerBuf[0]==0)
            {
                status.Text = "No FPGA module found\n";
                powerOff(); // to power off
                return;
            }

            // get module ID
            SerBuf[0] = (byte) cmds.GET_ID;
            transmit(1);
            recieve(5);
            if (SerBuf[0] == 44)
            {
                status.Text = string.Format("iceBLIP v{0:X1}, ", SerBuf[1]); 
                status.Text += string.Format("Flash ID {0:X02} {1:X02} {2:X02} \r\n", SerBuf[2], SerBuf[3], SerBuf[4]);
                status.Update();
            }
            else
            {
                status.Text = "Error getting Version\n";
                powerOff(); 
                return;
            }

            // erase sufficient 64k sectors
            int endPage = ((offset + len) >> 16) + 1;
            for (int page = (offset >> 16); page < endPage; page++)          
            {
                SerBuf[0] = (byte) cmds.ERASE_64K;
                SerBuf[1] = (byte) page;
                transmit(2);
                status.Text += string.Format("Erasing sector {0:X02}0000\r\n", page);
                status.Update();
                recieve(1);
            }

            // program the Flash
            int addr = offset;
            status.Text += "Programming ";
            status.Update();
            int cnt = 0;
            int endAddr = addr + len; // no flashsize check
            while (addr < endAddr)
            {
                SerBuf[0] = (byte)cmds.PROG_PAGE;
                SerBuf[1] = (byte)(addr >> 16);
                SerBuf[2] = (byte)(addr >> 8);
                SerBuf[3] = (byte)(addr);
                for (int x = 0; x < 256; x++) SerBuf[x + 4] = ProgBuf[addr++];
                transmit(260);
                recieve(4);
                if (SerBuf[0] != 0)
                {
                    status.Text += string.Format("\nProgram failed at {0:X06}, {1:X02} expected, {2:X02} read.\r\n", addr - 256 + SerBuf[1] - 4, SerBuf[2], SerBuf[3]);
                    powerOff();
                    return;
                }
                if (++cnt == 10)
                {
                    cnt = 0;
                    status.Text += ".";
                    status.Update();
                }
            }

            // Verify Flash
            if (verify.Checked == true)
            {
                addr = offset;
                status.Text += "\r\nVerifying   ";
                cnt = 0;
                while (addr < endAddr)
                {
                    SerBuf[0] = (byte)cmds.VERIFY_PAGE;
                    SerBuf[1] = (byte)(addr >> 16);
                    SerBuf[2] = (byte)(addr >> 8);
                    SerBuf[3] = (byte)addr;
                    for (int x = 0; x < 256; x++) SerBuf[x + 4] = ProgBuf[addr++];
                    transmit(260);
                    recieve(4);
                    if (SerBuf[0] > 0)
                    {
                        status.Text += string.Format("\nVerify failed at {0:X06}, {1:X02} expected, {2:X02} read.\r\n", addr - 256 + SerBuf[1] - 4, SerBuf[2], SerBuf[3]);
                        status.Text += string.Format("\nV {0:X02}, {1:X02}, {2:X02}, {3:X02}\r\n", SerBuf[0], SerBuf[1], SerBuf[2], SerBuf[3]);
                        powerOff();
                        return;
                    }
                    if (++cnt == 10)
                    {
                        cnt = 0;
                        status.Text += ".";
                        status.Update();
                    }
                }
            }

            releaseFPGA(powerHold.Checked);             // keep powered up when non-zero
            status.Text += "\r\nDone.\r\n";

        }

        private void offsetTxt_TextChanged(object sender, EventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("Offset", offsetTxt.Text);
        }

        private void verify_CheckedChanged(object sender, EventArgs e)
        {
            if(verify.Checked) Application.UserAppDataRegistry.SetValue("Verify", "1");
            else Application.UserAppDataRegistry.SetValue("Verify", "0");
        }

        private void powerHold_CheckedChanged(object sender, EventArgs e)
        {
            if (powerHold.Checked) Application.UserAppDataRegistry.SetValue("PowerHold", "1");
            else Application.UserAppDataRegistry.SetValue("PowerHold", "0");
        }
    }
}
